/* Copyright 2006 Oliver Madge
 * 
 * This file is part of RU-decoder.
 *
 * RU-decoder is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2 as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program in the file COPYING; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 *
 */


#ifndef COLUMNVECTOR_H
#define COLUMNVECTOR_H

#include "stdio.h"
#include <list>
#include <algorithm>

using namespace std;

/**
 *  \class columnVector
 * Column vector class.
 * More detailed description to follow here.
 */

class columnVector {
  public:

    /**
     * A constructor.
     * Create a columnVector of length h.
     * @param h an integer argument.
     * @see ~columnVector()
     */
    columnVector(int h);

    /**
     * A destructor.
     * Delete the vector and free its memory..
     * @see columnVector()
     */
    ~columnVector();

    void clear();

    /**
     * Copy another column vector to this. Both column vectors must be the same size.
     * @param input Column vector to clone.
     */
    void copy(const columnVector *input);

    /**
     * Add another column vector to this. Both column vectors must be the same size.
     * @param input Column vector to add.
     */
    void add(const columnVector *input);

    /**
     * Bitwise add (xor) another column vector to this. Both column vectors must be the same size.
     * @param input Column vector to bitwise add.
     */
    void bitwiseAdd(const columnVector *input);

    void bitwiseAdd(const columnVector *input, int offset, int length);

    /**
     * Subtract another column vector from this. Both column vectors must be the same size.
     * @param input Column vector to subtract.
     */
    void subtract(const columnVector *input);

    /**
     * Get a value from a specified row of the column vector.
     * @param index Row of the vector
     * @return Value from the vector
     */
    int get(int index);

    /**
     * Set a value of the column vector
     * @param index Row of the vector
     * @param value Value to set
     * @return Vector height
     */
    void push(int index, int value);

    /**
     * Swap two rows in the column vector.
     * @param i Row to swap
     * @param j Row to swap
     */
    void swapRows(int i, int j);


    /**
     * Get the height of the column vector.
     * @return Vector height
     */
    int getHeight() {return height;};

    /**
     * Display the column vector in a column.
     * @see prettyPrintTranspose()
     */
    void prettyPrint();

    /**
     * Display the column vector in a row.
     * @see prettyPrint()
     */
    void prettyPrintTranspose();
    void prettyPrintTransposeSort();

    bool inRange(int index);

  protected:
    /**
     * Array that stores the data.
     */
    int *data;

  private:
    int height;
};

#endif
