/* Copyright 2006 Oliver Madge
 * 
 * This file is part of RU-decoder.
 *
 * RU-decoder is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2 as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program in the file COPYING; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 *
 */

#ifndef DENSE_H
#define DENSE_H

#include "stdio.h"
#include <list>
#include "columnvector.h"
#include "random.h"
#include "util.h"


using namespace std;

/** \mainpage A dense matrix class
 * Provides modulo 2 manipulations of dense matrices.
 * More detailed description to follow here.
 */


#define BITS (sizeof(unsigned int)*8)

/**
 *  \class denseMatrix 
 * Sparse matrix class. Provides modulo 2 manipulations of sparse matrices.
 * More detailed description to follow here.
 */

class denseMatrix {
  public:
    
    /**
     * A constructor.
     * Create a denseMatrix of width w and height h.
     * @param w an integer argument.
     * @param h an integer argument.
     * @see ~denseMatrix()
     */
    denseMatrix(int w, int h);

    /**
     * A destructor.
     * Delete the sparse matrix and free its memory.
     */
    ~denseMatrix();

    /**
     * Set a row in the dense matrix.
     * @param where An integer argument. -1 is last row.
     * @param input Row data.
     */
    void setRow(int where, list<int> *input);

    /**
     * Clear the dense matrix 
     */
    void clear();

    /**
     * Invert the dense matrix, using spare rows as necessary
     */
    int slackInvert(columnVector*, columnVector*);


    /**
     * Display the sparse matrix in a nice way.
     * Prints each row padded with zeros, so it won't be readable for
     * huge sparse matrices.
     * @see prettyPrintTranspose()
     * @see debugPrint()
     * @see debugPrintTranspose()
     */
    
    void prettyPrint();
    
    /**
     * Display the sparse matrix in a form suitable for debugging.
     * Prints each row as a list of non-zero elements.
     * @see prettyPrint()
     * @see prettyPrintTranspose()
     * @see debugPrintTranspose()
     */
    void debugPrint();

    /**
     * Get the width of the sparse matrix.
     * @see getHeight()
     * @return The width
     */
    int getWidth() {return this->width;};

    /**
     * Get the height of the sparse matrix.
     * @see getWidth()
     * @return The height
     */
    int getHeight() {return this->height;};

  protected:


  private:

    unsigned int *data;

    int width, height;

};


#endif
