/* Copyright 2006 Oliver Madge
 * 
 * This file is part of RU-decoder.
 *
 * RU-decoder is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2 as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program in the file COPYING; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 *
 */

#include "sparse.h"

//Constructor
sparseMatrix::sparseMatrix(int w, int h){
  clearOpCounts();
 
  this->width=w;
  this->height=h;
  this->rows = new vector<list<int>* >;
  this->columns = new vector<list<int>* >;

  for(int i=0; i<this->height; i++){
    this->rows->push_back(new list<int>);
  }

  for(int i=0; i<this->width; i++){
    this->columns->push_back(new list<int>);
  }

}

//Destructor
sparseMatrix::~sparseMatrix(){
  for(int i=0;i<this->height; i++) delete (*this->rows)[i];
  for(int i=0;i<this->width; i++) delete (*this->columns)[i];

  if(this->rows != NULL) delete rows;
  if(this->columns != NULL) delete columns;
}

//Notes: very slow O(n^3)
//       may not be correct
int sparseMatrix::determinant(){

  int det=0, level[this->height];

  if(this->width!=this->height) { return -1; }

  int scratch[this->width];

  for(int i=0; i<this->width; i++) scratch[i]=0;

  int j=0;
  list<int>::iterator iter[this->height];
  iter[0] = (*this->rows)[0]->begin();

  level[0]=(*iter[0]);
  scratch[level[0]]=1;
  j++;
  iter[j] = (*this->rows)[j]->begin();

  //push a level
  while(true) {
    if(iter[j] == (*this->rows)[j]->end()) {
      //pop
      if(j==0) break;
      j--;
      scratch[(level[j])]=0;
      if(iter[j] != (*this->rows)[j]->end()) { iter[j]++;}
      continue;
    }

    if(scratch[*iter[j]]==1) {
      iter[j]++;
      continue;
    }
    
    if(j<this->height-1) {
      level[j]=(*iter[j]);
      scratch[level[j]]=1;
      j++;
      iter[j] = (*this->rows)[j]->begin();
    }
    else
    {
      if(iter[j]==(*this->rows)[j]->end()) printf("KJDKFJKJ\n"); //what does this signify?
      det++;
      j--;
      scratch[level[j]]=0;
      iter[j]++;
    }
  }

  return det%2;
}

//cheap way to calculate the determinant of an lower triangular matrix
int sparseMatrix::lTDeterminant(){
  for(int i=0; i<this->width; i++) {
    if((*this->columns)[i]->front()!=i) return 0;
  }

  return 1;
}

//cheap way to calculate the determinant of an upper triangular matrix
int sparseMatrix::uTDeterminant(){
  for(int i=0; i<this->height; i++) {
    if((*this->rows)[i]->front()!=i) return 0;
  }

  return 1;
}

////////////////////////////////////////////////////////////////////////
//  Element manipulations
////////////////////////////////////////////////////////////////////////

int sparseMatrix::insert(int i, int j){
  //check i/j are within bounds (1,1) is top left
  int ret=checkWithinBounds(i,j);
  if(ret!=0){
    fprintf(stderr, "Error: attempt to insert element out of bounds\n");
    return ret;
  }

  //add to the correct row list in order
  list<int>::iterator where_iter = (*this->rows)[i-1]->begin();
  while(where_iter != (*this->rows)[i-1]->end() && (*where_iter) < j-1) {
    where_iter++; // Search the list
  }

  if(where_iter != (*this->rows)[i-1]->end() && (*where_iter)==j-1) {
    //fprintf(stderr, "Error: element %i, %i already exists in rows\n", i, j);
    goto cols;
  }
  (*this->rows)[i-1]->insert(where_iter, j-1);

cols:
  //add to the correct column list in order
  where_iter = (*this->columns)[j-1]->begin();
  while(where_iter != (*this->columns)[j-1]->end() && (*where_iter) < i-1) where_iter++; // Search the list

  if(where_iter != (*this->columns)[j-1]->end() && (*where_iter)==i-1) {
    //fprintf(stderr, "Error: element %i, %i already exists in columns\n", i, j);
    return -1;
  }

  (*this->columns)[j-1]->insert(where_iter, i-1);

  return 0;
}

int sparseMatrix::remove(int j, int i){
  //check i/j are within bounds (1,1) is top left
  int ret=checkWithinBounds(i,j);
  if(ret!=0) return ret;

  //find element in rows
  list<int>::iterator where_iter = find((*this->rows)[j-1]->begin(), (*this->rows)[j-1]->end(), i-1);
  if(where_iter != (*this->rows)[j-1]->end()){
    (*this->rows)[j-1]->erase(where_iter);
  }
  else{
    //fprintf(stderr, "Attempt to erase non-existant element\n");
  }

  //find element in columns
  where_iter = find((*this->columns)[i-1]->begin(), (*this->columns)[i-1]->end(), j-1);
  if(where_iter != (*this->columns)[i-1]->end()){
    (*this->columns)[i-1]->erase(where_iter);
  }
  else{
    //fprintf(stderr, "Attempt to erase non-existant element\n");
  }

  return 0;
}



////////////////////////////////////////////////////////////////////////
//  Matrix manipulations
////////////////////////////////////////////////////////////////////////

void sparseMatrix::insertRow(int where, list<int> *input){
  //printf("Insert row not tested\n");

  if(where==-1) where = this->rows->size();

  if(where>this->rows->size()){
    fprintf(stderr, "Attempt to insert past the end\n");
    return;
  }

  vector<list<int>* >::iterator iter = this->rows->begin() + where;
  this->rows->insert(iter, input);
  this->height++;

  //account for the getPacketList() ops here
  this->binaryOps+=input->size();

  //TODO: flag columns as dirty
  //this->copyColumnsFromRow(where);
}

void sparseMatrix::replaceRow(int where, list<int> *input){
  //printf("Insert row not tested\n");

  vector<list<int>* >::iterator iter = this->rows->begin() + where;

  delete (*iter);

  (*iter) = input;

  //TODO: flag columns as dirty
  //this->copyColumnsFromRow(where);
}

void sparseMatrix::replaceRow(sparseMatrix *m2, int where, int srcrow){
  vector<list<int>* >::iterator iter = this->rows->begin() + where;
  (*iter) = *(m2->rows->begin()+srcrow);

  //TODO: flag columns as dirty
}

void sparseMatrix::addRow(int where, list<int> *input){

  if(where==-1) where = this->rows->size()-1;

  list<int>::iterator iter1=(*this->rows)[where]->begin();
  list<int>::iterator iter2=input->begin();

  while(iter1 != (*this->rows)[where]->end() && iter2 != input->end()) {
    if((*iter1)<(*iter2)) {
        iter1++;
    }
    else if((*iter1)>(*iter2)){
      //add to dest
      (*this->rows)[where]->insert(iter1, (*iter2));
      this->binaryOps++;
      iter2++;
    }
    else { //if((*iter1)==(*iter2)) {
      //delete this point
      iter1=(*this->rows)[where]->erase(iter1);
      this->binaryOps++;
      iter2++;
    }
  }

  if(iter1 == (*this->rows)[where]->end()) {
    //copy remainder of row from m2
    while(iter2 != input->end()){
      //dest->insert((*iter2)+1, i+1);
      (*this->rows)[where]->insert(iter1, (*iter2));
      this->binaryOps++;
      iter2++;
    }
  }
  else if(iter2 == input->end()) {
    //do nothing
  }

  //TODO: flag columns as dirty
}

void sparseMatrix::addRow(int where, int where2){
  this->addRow(where2, (*this->rows)[where]);
}

void sparseMatrix::insertColumn(int where, list<int> *input){
  //printf("Insert column not tested\n");

  if(where>this->columns->size()){
    fprintf(stderr, "Attempt to insert past the end\n");
    return;
  }

  vector<list<int>* >::iterator iter = this->columns->begin() + where;
  this->columns->insert(iter, input);

  this->width++;

  //TODO: flag rows as dirty
  this->copyRowsFromColumn(where);
}

void sparseMatrix::clear(){
  clearRows();
  clearColumns();
}

int sparseMatrix::blankColumns() {
  for(int i=0; i<this->columns->size(); i++) {
    if((*this->columns)[i]->size()==0) return i;
  }
  return -1;

  int total = 0;
  int colTotal[this->height];
  for(int i=0; i<this->height; i++) colTotal[i]=0;
  for(int i=0; i<this->columns->size(); i++) {
    //printf("%i, ", (*this->columns)[i]->size());
    int tmp = (*this->columns)[i]->size();
    colTotal[tmp]++;
    total += tmp;    //if((*this->columns)[i]->size()==0) return i;
  }
  /*long double ans = (double)total/(double) this->columns->size();
  printf("rho'(1)=%Le\n", ans);*/
  
  for(int i=0; i<50; i++) printf("%i ",colTotal[i]);
  printf("\n");
  return -1;
}

void sparseMatrix::transpose(){
  //swap *rows and *columns

  vector<list<int>* > *tmpRows;
  tmpRows = rows;
  rows = columns;
  columns = tmpRows;
  
  int tmpH = this->height;
  this->height = this->width;
  this->width = tmpH;

}

void sparseMatrix::swapRows(int i, int j){

  if(i>this->height || j>this->height || j==i) return;

  list<int>* tmpRow = (*this->rows)[i-1];
  (*this->rows)[i-1] = (*this->rows)[j-1];
  (*this->rows)[j-1] = tmpRow;

  //printf("row swap\n");

  //TODO: flag columns as dirty
}

void sparseMatrix::swapRowsSync(int i, int j){

  if(i>this->height || j>this->height || j==i) return;

  //printf("srow swap\n");

  //now walk the rows, and fix the columns
  
  int row1 = i<j ? i-1 : j-1;
  int row2 = i<j ? j-1 : i-1;

  //printf("Rows %i, %i\n",row1, row2);

  list<int>::iterator iter=(*this->rows)[row1]->begin();
  list<int>::iterator iter2=(*this->rows)[row2]->begin();


  while(iter!=(*this->rows)[row1]->end() && iter2!=(*this->rows)[row2]->end()) {
    if(*iter==*iter2) {iter++; iter2++; }//both in same column so do nothing
    else if(*iter2<*iter) {
      //fix column *iter2
      //go into col[*iter2], insert item at row1, remove item at row2
      list<int>::iterator colIter=(*this->columns)[*iter2]->begin();
      while(colIter!=(*this->columns)[*iter2]->end() && *colIter<row1) colIter++;
      //printf("colIter=%i, row1=%i\n", *colIter, row1);
      //colIter--;
      //printf("now colIter=%i, row1=%i\n", *colIter, row1);
      (*this->columns)[*iter]->insert(colIter, row1);
      while(colIter!=(*this->columns)[*iter2]->end() && *colIter<row2) colIter++;
      //if we didn't find it then something went badly wrong
      //printf("are they equal? %i, %i\n", row2, *colIter);
      colIter = (*this->columns)[*iter2]->erase(colIter);
      iter2++;
    }
    else { //*iter<*iter2
      //fix column *iter
      //go into col[*iter], remove item at row1, insert item at row2
      list<int>::iterator colIter=(*this->columns)[*iter]->begin();
      while(colIter!=(*this->columns)[*iter]->end() && *colIter<row1) colIter++;
      //if we didn't find it then something went badly wrong
      //printf("iter2: are they equal? %i, %i\n", row1, *colIter);
      colIter = (*this->columns)[*iter]->erase(colIter);
      while(colIter!=(*this->columns)[*iter]->end() && *colIter<row2) colIter++;
      //colIter--;
      (*this->columns)[*iter]->insert(colIter, row2);
      iter++;
    }
    //this->prettyPrintTranspose();
    //this->debugPrintTranspose();
    //printf("\n");
  }

  while(iter!=(*this->rows)[row1]->end()) {
    //copy the rest of *iter into the correct place
    list<int>::iterator colIter=(*this->columns)[*iter]->begin();
      while(colIter!=(*this->columns)[*iter]->end() && *colIter<row1) colIter++;
      //if we didn't find it then something went badly wrong
      //printf("iter2: are they equal? %i, %i\n", row1, *colIter);
      colIter = (*this->columns)[*iter]->erase(colIter);
      while(colIter!=(*this->columns)[*iter]->end() && *colIter<row2) colIter++;
      //colIter--;
      (*this->columns)[*iter]->insert(colIter, row2);

        
    iter++;
  }
  
  while(iter2!=(*this->rows)[row2]->end()) {
    //copy the rest of *iter2 into the correct place
    
    list<int>::iterator colIter=(*this->columns)[*iter2]->begin();
    while(colIter!=(*this->columns)[*iter2]->end() && *colIter<row1) colIter++;
    //printf("colIter=%i, row1=%i\n", *colIter, row1);
    //colIter--;
    //printf("now colIter=%i, row1=%i\n", *colIter, row1);
    (*this->columns)[*iter]->insert(colIter, row1);
    while(colIter!=(*this->columns)[*iter2]->end() && *colIter<row2) colIter++;
    //if we didn't find it then something went badly wrong
    //printf("are they equal? %i, %i\n", row2, *colIter);
    colIter = (*this->columns)[*iter2]->erase(colIter);

    iter2++;
  }

  //printf("Leaving\n");
  
  list<int>* tmpRow = (*this->rows)[i-1];
  (*this->rows)[i-1] = (*this->rows)[j-1];
  (*this->rows)[j-1] = tmpRow;
  
}

void sparseMatrix::swapColumns(int i, int j){
  //printf("Column swapping not implemented\n");

  if(i>this->width || j>this->width || j==i || i<1 || j<1) {
    //printf("Column swap out of range\n");
    return;
  }


  list<int>* tmpColumn = (*this->columns)[i-1];

  (*this->columns)[i-1] = (*this->columns)[j-1];

  (*this->columns)[j-1] = tmpColumn;

  //printf("col swap\n");

  //TODO: flag rows as dirty
}

int sparseMatrix::triangulate5(columnVector* col, columnVector* ans, Random *rand, int stop){
  int sizeOfT=0, delta=0;
  int i;
  int bottomlimit = this->height-1;

  int sizemin=this->width;

  int colSizeCache[this->width];
  int minRowWeightCache[this->width];
  int sizeOneCache[this->width], numOnes=0;

  int colWhereCache[this->width];
  int rowSizeCache[this->height];

  int colOrder[this->width], invColOrder[this->width];
  int rowOrder[this->height], invRowOrder[this->height];
  
  for(i=0; i<this->height; i++) {
    rowOrder[i]=i; invRowOrder[i]=i;
    rowSizeCache[i]=(*this->rows)[i]->size();
  }
  
  
  for(i=0; i<this->width; i++) {
    colOrder[i]=i; invColOrder[i]=i;
    colSizeCache[i]=0; colWhereCache[i]=0;
    list<int>::iterator iter=(*this->columns)[i]->begin();
    int tmp, j=this->width;
    while(iter!=(*this->columns)[i]->end()){
      tmp = *iter;
      colSizeCache[i]++;
      colWhereCache[i]+=tmp;
      if(rowSizeCache[tmp]<j) j = rowSizeCache[tmp]; 
      iter++;
    }
    minRowWeightCache[i]=j;
    //minRowWeightCache[i]=0;
    if(colSizeCache[i]==1) {
      //add it to the sizeOneCache[];
      sizeOneCache[numOnes] = i;
      numOnes++;
    }
  }
  //could assume height>=width here for small optimisation
  
  //FIXME: combine this loop with the previous one for more speedup
  for(int i=0; i<this->width; i++) { //for each column
    //find the number of elements in this column
    int currentCol=colOrder[i];
    int size = colSizeCache[currentCol];
    if(size>1 && size<sizemin) sizemin = size; 
  }

  //printf("%i\n", sizemin);

  /* Algorithm C */
  /* Select one of the lowest degree columns
   * that is also connected to a high degree row
   * as a starting point
   */
  
  //bottomlimit -= splitDegreeRows(col, 0, bottomlimit, rowOrder, invRowOrder, colOrder, invColOrder, colSizeCache, rowSizeCache, colWhereCache);
  
  //for(i=0; i<this->height; i++) {colCheck[i]=-1;}

  //int delta2=1;
  //enter main loop, extend diagonal
  //printf("Starting\n");
  do {
    delta = 0; 
    //find all degree one columns in the reduced matrix (between toplimit and bottomlimit)
    //printf("blimit=%i\n", bottomlimit);
    
    /*for(i=sizeOfT; i<this->width; i++) {
      int currentCol = colOrder[i]; //correct
      //record the position of the element (this locates the column)
      int columnSize = colSizeCache[currentCol];
    */
    while(numOnes>0) {
      numOnes--;
      int currentCol = sizeOneCache[numOnes];
      int columnSize = colSizeCache[currentCol];
      //printf("Col %i\n", currentCol);
      //if(columnSize!=1) printf("WTF %i\n", columnSize);
      if(columnSize==1) { //length=1
        int tmpRow = invRowOrder[colWhereCache[currentCol]];
        
        //swap tmpRow to the top
        
        //top is at sizeOfT
        //current row is at tmpRow
        //printf("swapping row %i with %i\n", tmpRow, sizeOfT);
        int swappingRow = rowOrder[tmpRow];
        invRowOrder[rowOrder[tmpRow]] = sizeOfT;
        invRowOrder[rowOrder[sizeOfT]] = tmpRow;

        rowOrder[tmpRow] = rowOrder[sizeOfT];
        rowOrder[sizeOfT] = swappingRow;

        //walk tmpRow, subtracting 1 from each element
        list<int>::iterator iter = (*this->rows)[swappingRow]->begin();
        while(iter != (*this->rows)[swappingRow]->end()) {
          colSizeCache[*iter]--;
          if(colSizeCache[*iter]==1) {
            //add it to the sizeOneCache[];
            //printf("here %i\n", *iter);
            sizeOneCache[numOnes] = *iter;
            numOnes++;
          }
          if(colSizeCache[*iter]<sizemin && colSizeCache[*iter]>1) sizemin = colSizeCache[*iter];
          colWhereCache[*iter]-=swappingRow;
          if(minRowWeightCache[*iter]==rowSizeCache[swappingRow]) minRowWeightCache[*iter]=0;
          iter++;
        }

          //minRowWeightCache[*iter]=0;
        col->swapRows(tmpRow, sizeOfT);

        this->swapOps++;
       
        //swap currentCol to the top left

        //int swappingCol = colOrder[i]; //currentCol
        i=invColOrder[currentCol];
        //printf("swapping col %i with %i\n", currentCol, sizeOfT);
        //invColOrder[colOrder[sizeOfT]] = invColOrder[colOrder[i]];
        invColOrder[colOrder[sizeOfT]] = invColOrder[currentCol];
        invColOrder[currentCol] = sizeOfT;
        colOrder[i] = colOrder[sizeOfT];
        colOrder[sizeOfT] = currentCol;

        ans->swapRows(i, sizeOfT);
        
        //continue to the next iteration
        //colCheck[tmpRow] = delta;
        //lOneRows[delta] = i;
        //knownCol[delta] = tmpRow;
        sizeOfT++;
        delta++;

        //if(sizeOfT>=stop) break;
        //break;
      }

    }
    //if(sizeOfT>=stop) break;

    if(delta==0) {
      //if(delta2==0) printf("Hmm\n");
      //delta2=0;
      //printf("a\n");
      if(sizeOfT>=bottomlimit) break;
      
      //actually try and split a degree \geq two into junk and not
      
      delta = splitDegreeRows(col, sizeOfT, bottomlimit, rowOrder, invRowOrder, colOrder, invColOrder, colSizeCache, rowSizeCache, colWhereCache, sizeOneCache, &numOnes, minRowWeightCache, &sizemin);
      //printf("numOnes = %i\n", numOnes);
      if(delta!=0) {
        //printf("%i\n", delta);
        bottomlimit -= delta;
      }
      continue;
      
    }
    //else delta2=1;
    
  } while (delta>0);

  //printf("Done\n");
  //now put everything back in order :S

  //make copy of rows vector
  vector<list<int>* > *rowsCopy = new vector<list<int>* >;

  for(i=0; i<this->rows->size(); i++) {
    rowsCopy->push_back((*this->rows)[i]);
  }
  /*
  for(i=0; i<this->width;i++) {
    printf("%i, ", invColOrder[i]);
  }
  printf("\n");
  */
  
  for(i=0; i<this->height; i++) {
    //put correct row according to rowOrder[i] in row, get pointers from copy
    (*this->rows)[i] = (*rowsCopy)[rowOrder[i]];
    
    
    //go through row and transform each entry as val = invColOrder[val]
    //
    
    list<int>::iterator iter = (*this->rows)[i]->begin();
    while(iter != (*this->rows)[i]->end()) {
      *iter = invColOrder[*iter];
      iter++;
    }
    //sort row
    (*this->rows)[i]->sort();
    
  }

  delete rowsCopy;

  //coordPrint();
  //debugPrint();
  //prettyPrint();

  copyColumnsFromRows();

  return sizeOfT;
}



int sparseMatrix::triangulate4(columnVector* col, columnVector* ans, Random *rand){
  int sizeOfT=0, delta=0;
  int knownCol[this->height], colCheck[this->height], knowns, lOneRows[this->width];
  int i;
  int bottomlimit = this->height-1;

  int colOrder[this->width], invColOrder[this->width];
  int rowOrder[this->height], invRowOrder[this->height];

  printf("Warning: triangulate4(...) is deprecated, use triangulate5(...) instead\n");
  printf("This will crash\n");

  for(i=0; i<this->width; i++) { colOrder[i]=i; invColOrder[i]=i;}
  //could assume height>=width here for small optimisation
  for(i=0; i<this->height; i++) { rowOrder[i]=i; invRowOrder[i]=i;}

  /* Algorithm C */
  /* Select one of the lowest degree columns
   * that is also connected to a high degree row
   * as a starting point
   */
  
  printf("This is going to break\n");
  bottomlimit -= splitDegreeRows(col, 0, bottomlimit, rowOrder, invRowOrder, colOrder, invColOrder, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
  
  for(i=0; i<this->height; i++) {colCheck[i]=-1;}

  //enter main loop, extend diagonal
  do {
    delta = 0; knowns=0;
    //find all degree one columns in the reduced matrix (between toplimit and bottomlimit)
    //printf("blimit=%i\n", bottomlimit);
    
    for(i=sizeOfT; i<this->width; i++) {
      int currentCol = colOrder[i]; //correct
      //record the position of the element (this locates the column)
      list<int>::iterator iter = (*this->columns)[currentCol]->begin();
      int columnSize = 0, tmpRow;
      while(iter != (*this->columns)[currentCol]->end()) {
        if(invRowOrder[*iter]>=sizeOfT && invRowOrder[*iter]<=bottomlimit){ 
          columnSize++;
          //record where we are in case the column is of weight 1
          tmpRow = invRowOrder[*iter];
        }
        iter++;
      }
      
      if(columnSize==0) { //length=0
        if(sizeOfT==0) {
          fprintf(stderr,"Triangulation failed, blank column detected\n");
          //return -i-2;
          continue;
        }
        //printf("deg 0\n");
        continue;
      }
      if(columnSize==1) { //length=1
        //swap tmpRow to the top
        
        //top is at sizeOfT
        //current row is at tmpRow
        //printf("swapping row %i with %i\n", tmpRow, sizeOfT);
        int swappingRow = rowOrder[tmpRow];
        invRowOrder[rowOrder[tmpRow]] = sizeOfT;
        invRowOrder[rowOrder[sizeOfT]] = tmpRow;

        rowOrder[tmpRow] = rowOrder[sizeOfT];
        rowOrder[sizeOfT] = swappingRow;

        col->swapRows(tmpRow, sizeOfT);
       
        //swap currentCol to the top left

        //int swappingCol = colOrder[i]; //currentCol
        //printf("swapping col %i with %i\n", i, sizeOfT);
        invColOrder[colOrder[sizeOfT]] = invColOrder[colOrder[i]];
        invColOrder[currentCol] = sizeOfT;
        colOrder[i] = colOrder[sizeOfT];
        colOrder[sizeOfT] = currentCol;

        ans->swapRows(i, sizeOfT);

        
        //continue to the next iteration
        //colCheck[tmpRow] = delta;
        //lOneRows[delta] = i;
        //knownCol[delta] = tmpRow;
        sizeOfT++;
        delta++;
        break;
      }
    }

    /*
    for(i=0; i<this->width;i++) {
      printf("%i, ", colOrder[i]);
    }
    printf("\n");
    for(i=0; i<this->width;i++) {
      printf("%i, ", invColOrder[i]);
    }
    
    printf("\n");
    for(i=0; i<this->height;i++) {
      printf("%i, ", rowOrder[i]);
    }
    printf("\n");
    for(i=0; i<this->height;i++) {
      printf("%i, ", invRowOrder[i]);
    }
    printf("\n");
    */


    if(delta==0) {
      if(sizeOfT>=bottomlimit) break;
      
      //actually try and split a degree \geq two into junk and not
      
      delta = splitDegreeRows(col, sizeOfT, bottomlimit, rowOrder, invRowOrder, colOrder, invColOrder, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
      if(delta!=0) {
        bottomlimit -= delta;
      }
      continue;
      
    }
    
  } while (delta>0);


  //now put everything back in order :S

  //make copy of rows vector
  vector<list<int>* > *rowsCopy = new vector<list<int>* >;

  for(i=0; i<this->rows->size(); i++) {
    rowsCopy->push_back((*this->rows)[i]);
  }
  /*
  for(i=0; i<this->width;i++) {
    printf("%i, ", invColOrder[i]);
  }
  printf("\n");
  */
  
  for(i=0; i<this->height; i++) {
    //put correct row according to rowOrder[i] in row, get pointers from copy
    (*this->rows)[i] = (*rowsCopy)[rowOrder[i]];
    
    
    //go through row and transform each entry as val = invColOrder[val]
    //
    
    list<int>::iterator iter = (*this->rows)[i]->begin();
    while(iter != (*this->rows)[i]->end()) {
      *iter = invColOrder[*iter];
      iter++;
    }
    //sort row
    (*this->rows)[i]->sort();
    
  }

  delete rowsCopy;

  //debugPrint();
  //prettyPrint();

  copyColumnsFromRows();

  return sizeOfT;
}



int sparseMatrix::triangulate3(columnVector* col, columnVector* ans, Random *rand, sparseMatrix* m2){
  int sizeOfT=0, delta=0;
  int knownCol[this->height], colCheck[this->height], knowns, lOneRows[this->width];
  int i;
  int bottomlimit = this->height-1;
  //printf("botl=%i\n", bottomlimit);

  //for(i=0; i<this->height; i++) {
  //  colCheck[i]=-1;
  //}

  //select initial rows

  //printf("Selecting %i columns initially\n", selection);

  /* Algorithm C */
  /* Select one of the lowest degree columns
   * that is also connected to a high degree row
   */
   
  bottomlimit -= splitDegreeRows(col, 0, bottomlimit);
  
  for(i=0; i<this->height; i++) {colCheck[i]=-1;}

  //enter main loop, extend diagonal
  do {
    delta = 0; knowns=0;
    //find all degree one columns in the reduced matrix (between toplimit and bottomlimit)
    for(i=sizeOfT; i<this->width; i++) {
      //record the position of the element (this locates the column)
      list<int>::iterator iter = (*this->columns)[i]->begin();
      while(sizeOfT!=0 && iter != (*this->columns)[i]->end() && (*iter)<sizeOfT) {
        iter++;
      }
      
      if(iter==(*this->columns)[i]->end()) { //length=0
        if(sizeOfT==0) {
          fprintf(stderr,"Triangulation failed, blank column detected\n");
          //return -i-2;
        }
        //printf("deg 0\n");
        continue;
      }
      int tmpRow = *iter;
      iter++;
      if(tmpRow<=bottomlimit && (iter==(*this->columns)[i]->end() || (*iter)>bottomlimit) ) { //length=1
        //check we don't already have this column
        if(colCheck[tmpRow]!=-1) {
          //printf("deg 1\n");
          continue;
        }
        //printf("swapping row %i with %i\n", tmpRow, delta+sizeOfT);

        //printf("swapping col %i with %i\n", i, delta+sizeOfT);

        colCheck[tmpRow] = delta;
        lOneRows[delta] = i;
        knownCol[delta] = tmpRow;
        delta++;
        knowns++;
      }
      //printf("bl=%i\n", bottomlimit);
    }

    if(delta==0) {
      if(sizeOfT>=bottomlimit) break;
      //break;
      //actually try and split a degree two into junk and not
      
      delta = splitDegreeRows(col, sizeOfT, bottomlimit);
      if(delta!=0) {
        bottomlimit -= delta;
        //printf("bl=%i\n", bottomlimit);
      }
      continue;
      
    }

    i=0;
    while(i<knowns){
      this->swapColumns(lOneRows[i]+1, i+1 + sizeOfT);
      ans->swapRows(lOneRows[i], i + sizeOfT);
      i++;
    }
    this->copyRowsFromColumns();

    //swap correct columns
    i=0;
    while(i<knowns){

      //column we're swapping into doesn't contain something, or *us*
      //(ie. we're already in the right column)
      colCheck[knownCol[i]] = -1;
      if(colCheck[i + sizeOfT] !=-1 && colCheck[i + sizeOfT]!=i + sizeOfT) {
        //tell it where it went
        colCheck[knownCol[i]] = colCheck[i + sizeOfT];
        knownCol[colCheck[i + sizeOfT]] = knownCol[i];
      }
      colCheck[i + sizeOfT] = -1;

      this->swapRows(knownCol[i]+1, i+1 + sizeOfT);
      col->swapRows(knownCol[i], i + sizeOfT);

      i++;
    }
    this->copyColumnsFromRows();

    sizeOfT += delta;
    
  } while (delta>0);

  return sizeOfT;
}

int sparseMatrix::colSizeLim(int tlimit, int blimit, int i) {
    list<int>::iterator iter = (*this->columns)[i]->begin();
    while(tlimit!=0 && iter != (*this->columns)[i]->end() && (*iter)<tlimit) {
      iter++;
    }
    int size=0;
    while(iter != (*this->columns)[i]->end() && (*iter)<=blimit) {
      size++;
      iter++;
    }
    return size;
}

int sparseMatrix::splitDegreeRows(columnVector *col, int tlimit, int blimit, int *rowOrder, int *invRowOrder, int *colOrder, int *invColOrder, int *colSizeCache, int *rowSizeCache, int *colWhereCache, int *sizeOneCache, int *numOnes, int *minRowWeightCache, int *sizemin) {
  int knowns=0, count=0;

  //int sizemin=this->width;

  //FIXME: cache sizemin

  //printf("Sizemin = %i\n", sizemin);

  //could speed up massively here by just choosing a random one
  //FIXME: speed this bit up
  int rowmax=0, where=-1, where2=0, colmin=this->height, colmax=0;
  //printf("%i\n",count);
  //if count>100 then choose a random one
  //otherwise actually check
  /*if(count>100) {
    where=count/2;
  }
  else  { */
try_again:
  //printf("Sizemin: %i\n", *sizemin);
    for(int i=tlimit; i<this->width; i++) { //for each column
      int currentCol = colOrder[i];
      if(colSizeCache[currentCol]==*sizemin){// || colSizeCache[currentCol]==sizemin+1 || colSizeCache[currentCol]==sizemin+2){

        //easy speedup
        /*if(count>400) {
          //where=currentCol; break;
          count--;
          continue;
        }*/
        //walk the column, counting the length of each connected row
        //assumption: all 0s to the left of this rows tlimit (if the
        //triangulation works as expected this is true)
        //printf("%i \n", currentCol);
        int j;
        if(minRowWeightCache[currentCol]!=0) {
          //printf("Cache hit\n");
          j = minRowWeightCache[currentCol]; 
        }
        else {
          //printf("Cache miss\n");
        j=this->width;
        list<int>::iterator iter=(*this->columns)[currentCol]->begin();

        //if(minRowWeightCache[currentCol]!=0) printf("Cache hit\n");
        //minRowWeightCache[currentCol] = 1;

        while(iter != (*this->columns)[currentCol]->end()) {

          if(invRowOrder[*iter]>=tlimit && invRowOrder[*iter]<=blimit) {
            //printf("Row cache %i\n", *iter);
            int tmp=0;
            tmp = rowSizeCache[*iter];

            //find the maximum minimum row weight
            if(tmp<j) {
              //printf("%i\n", tmp);
              j=tmp;
              //where2=*iter;
            }
          }
          iter++;
        }
        minRowWeightCache[currentCol] = j;
        //minRowWeightCache[currentCol] = 0;
        }
        if(j>rowmax){
          rowmax = j; 
          where = currentCol;
          //colmax = 0;
          //colmin = this->width;
        }
        /*if(j==rowmax) {
          if(colSizeCache[currentCol]>colmax) {
            colmax = colSizeCache[currentCol];
            where=currentCol;
          }
          if(colSizeCache[currentCol]<colmin) {
            colmin = colSizeCache[currentCol];
            where2=currentCol;
          }
        }*/

      }
    }

  if(where==-1) { //rebuild minsize
    //printf("Eep\n");
    *sizemin=this->width;
    for(int i=tlimit; i<this->width; i++) { //for each column
      //find the number of elements in this column
      int currentCol=colOrder[i];
      int size = colSizeCache[currentCol];
      if(size>1 && size<*sizemin) *sizemin = size; 
    }
    
    goto try_again;
  }

   // if(rowmax<16) { where = where2;}

  //printf("smin=%i, rmax=%i, where=%i\n", sizemin, rowmax, where);
  knowns = *sizemin-1;
  //printf("k=%i\n",knowns);
  list<int>::iterator iter=(*this->columns)[where]->end();
  //while(tlimit!=0 && iter != (*this->columns)[where]->end() && (*iter)<tlimit) {
  //  iter++;
  //}
  for(int i=0; i<knowns; i++) {
    iter--;
    //printf("swapping %i\n",*iter);
    if(invRowOrder[*iter]>=tlimit && invRowOrder[*iter]<=blimit) {
      //swap invRowOrder[*iter] with blimit-i
      int tmpRow = invRowOrder[*iter];
      int swappingRow = rowOrder[tmpRow];
      //printf("Swapping: %i, %i; knowns=%i\n", swappingRow+1,blimit-i+1, knowns);
      //FIXME: bug here, maybe fixed
      invRowOrder[rowOrder[blimit-i]] = invRowOrder[rowOrder[tmpRow]];
      invRowOrder[swappingRow] = blimit-i;
      rowOrder[tmpRow] = rowOrder[blimit-i];
      rowOrder[blimit-i] = swappingRow;

      if(colSizeCache != NULL) {
        //walk tmpRow, subtracting 1 from each element
        list<int>::iterator iter2 = (*this->rows)[swappingRow]->begin();
        while(iter2 != (*this->rows)[swappingRow]->end()) {
          int tmp2 = *iter2;
          colSizeCache[tmp2]--;
          if(colSizeCache[tmp2]==1) {
            //add it to the sizeOneCache[];
            //printf("here s, %i\n", *numOnes);
            sizeOneCache[*numOnes] = tmp2;
            (*numOnes)++;
          }
          else if(colSizeCache[tmp2]<*sizemin && colSizeCache[tmp2]>1) *sizemin = colSizeCache[tmp2];
          
          colWhereCache[tmp2]-=swappingRow;
          if(minRowWeightCache[tmp2]==rowSizeCache[swappingRow]) minRowWeightCache[tmp2]=0;
          iter2++;
        }

      }

      //printf("p\n");
      col->swapRows(tmpRow, blimit - i);
      
      this->swapOps++;

      //printf("p\n");
    }
    else {i--;}
  }

  //printf("knowns=%i\n", knowns);
  //copy into array

  //not connected to degree one rows (there are none)
  //so push them to the bottom edge
  //assume that the knownCol[] are ordered
  //int i=0, tknowns = knowns;
  //while(knowns>0){
  //  knowns--;
  //  this->swapRowsSync(knownCol[knowns]+1,blimit-i+1);
  //  //this->swapRows(knownCol[knowns]+1,blimit-i);
  //  col->swapRows(knownCol[knowns],blimit-1-i);
  //  i++;
  //}
  //printf("limits: %i, %i\n",tlimit, blimit);
  //this->copyColumnsFromRows(tlimit, blimit);
  //this->copyColumnsFromRows();
  //printf("again\n");

  return knowns;

}

int sparseMatrix::splitDegreeRows(columnVector *col, int tlimit, int blimit) {
  int knowns=0;
  int knownCol[this->width];
  int sizemin=this->width;
  for(int i=tlimit; i<this->width; i++) { //for each column
    //find the number of elements in this column
    
    int size=colSizeLim(tlimit, blimit, i);
    knownCol[i] = size;
    if(size>0 && size<sizemin) sizemin = size;
  }

  //could speed up massively here by just choosing a random one
  int rowmax=0, where=0, where2=0;
  for(int i=tlimit; i<this->width; i++) { //for each column
    if(knownCol[i]==sizemin){

      //walk the column, counting the length of each connected row
      //assumption: all 0s to the left of this rows tlimit (if the
      //triangulation works as expected this is true)
      list<int>::iterator iter=(*this->columns)[i]->begin();
      while(tlimit!=0 && iter != (*this->columns)[i]->end() && (*iter)<tlimit) {
        iter++;
      }
      int j=this->width;
      while(iter != (*this->columns)[i]->end() && (*iter)<=blimit) {
        
        int tmp =(*this->rows)[*iter]->size();
        //find the maximum minimum row weight
        if(tmp<j) { 
          j=tmp; 
          where2=*iter;
        }
        iter++;
      }
      if(j>rowmax) {
        rowmax = j;
        where=i;
      }

    }
  }

  //printf("smin=%i, rmax=%i, where=%i\n", sizemin, rowmax, where);
  knowns = sizemin-1;
  list<int>::iterator iter=(*this->columns)[where]->begin();
  while(tlimit!=0 && iter != (*this->columns)[where]->end() && (*iter)<tlimit) {
    iter++;
  }
  iter++;
  for(int i=0; i<knowns; i++) {
    //printf("swapping %i\n",*iter);
    
    knownCol[i] = *iter;
    iter++;
  }

  //printf("knowns=%i\n", knowns);
  //copy into array

  //not connected to degree one rows (there are none)
  //so push them to the bottom edge
  //assume that the knownCol[] are ordered
  int i=0, tknowns = knowns;
  //printf("%i, ", knowns);
  while(knowns>0){
    knowns--;
    //this->swapRowsSync(knownCol[knowns]+1,blimit-i);
    //printf("Swapping: %i, %i\n", knownCol[knowns]+1,blimit-i+1);
    this->swapRows(knownCol[knowns]+1,blimit-i+1);
    col->swapRows(knownCol[knowns],blimit-i);
    i++;
  }
  //printf("limits: %i, %i\n",tlimit, blimit);
  //this->copyColumnsFromRows(tlimit, blimit);
  this->copyColumnsFromRows();
  //printf("again\n");

  return tknowns;

}

int sparseMatrix::triangulate2(columnVector* col, columnVector* ans, Random *rand, sparseMatrix* m2){
  int sizeOfT=0, delta=0;
  int knownCol[this->width], colCheck[this->width], knowns, lOneRows[this->height];
  double alpha = 0.25, w[this->width];
  int i;
  int rightlimit = this->width-1;

  

  for(i=0; i<this->width; i++) {
    //printf("%i ",(*this->columns)[i]->size());
    colCheck[i]=-1;
    w[i] = 0.0; 
  }
  //printf("\n");


  /*
  w[2] = 0.00;
  w[3] = 0.00;
  w[6] = 0.000;

  w[12] = 0.09;

  w[94] = 0.09;
  w[100] = 0.09;
  w[104] = 0.02;
  */ 
  w[2] = 0.7;
  w[20] = 0.8;
  w[22] = 0.2;
  w[21] = 0.2;

  w[4] = 0.02;
  w[70] = 0.01;
  w[111] = 0.05;
  w[148] = 0.2;

  //select initial rows

  //printf("Selecting %i columns initially\n", selection);

  /* Algorithm A */
  /*knowns=0;
  for(i=0; i<this->width; i++) {
    if(rand->randomd()<alpha){
      knownCol[knowns] = i;
      knowns++;
    }
  }*/

  /* Algorithm B */
  knowns=0;
  for(i=0; i<this->height; i++) { //for each rows
    //find the number of elements in this row
    int size = (*this->rows)[i]->size();
    if(rand->randomd()<w[size]){
      w[size]=0.0;
      int where = rand->randoml(0, size-1);
      //walk the row, adding each element to known columns
      list<int>::iterator iter=(*this->rows)[i]->begin();
      int j=0;
      while(iter != (*this->rows)[i]->end()) {
        if(j!=where && colCheck[*iter]==-1) {
          knownCol[knowns] = (*iter);
          knowns++;
          colCheck[*iter] = (*iter);
        }
        //printf("bleh, %i\n", i);
        iter++; j++;
      }
    }
  }

  /*
  int selection = (long)((double)this->width*alpha);
  i=this->width;

  list<int> *output = new list<int>();
  list<int>::iterator iter;

  int num;
  while(selection>0){
    num = rand->randoml(0,i-1);
    iter = output->begin();
    while(iter!=output->end() && (*iter)<=num){
      iter++; num++;
    }
    output->insert(iter, num);
    i--;
    selection--;
  }

  iter = output->begin();
  i=0;
  while(iter!=output->end()){
    knownCol[i] = (*iter); iter++; i++;
  }

  knowns = i;*/
  //printf("knowns=%i\n", knowns);
  //copy into array

  //not connected to degree one rows (there are none)
  //so push them to the right edge
  //assume that the knownCol() are ordered
  i=0;
  rightlimit -= knowns;
  while(knowns>0){
    knowns--;
    this->swapColumns(knownCol[knowns]+1,this->width-i);
    ans->swapRows(knownCol[knowns],this->width-i-1);
    i++;
  }
  this->copyRowsFromColumns();

  //printf("done");
  
  for(i=0; i<this->width; i++) {colCheck[i]=-1;}

  //enter main loop, extend diagonal
  do {
    delta = 0; knowns=0;
    //find all degree one rows in the reduced matrix (between leftlimit and rightlimit)
    for(i=sizeOfT; i<this->height; i++) {
      //record the position of the element (this locates the column)
      list<int>::iterator iter = (*this->rows)[i]->begin();
      while(sizeOfT!=0 && iter != (*this->rows)[i]->end() && (*iter)<sizeOfT) iter++;
      
      if(iter==(*this->rows)[i]->end()) { //length=0
        if(sizeOfT==0) {
          printf("Triangulation failed, blank row detected\n");
          return -i-2;
        }
        continue;
      }
      int tmpCol = (*iter);
      iter++;
      if(tmpCol<=rightlimit && (iter==(*this->rows)[i]->end() || (*iter)>rightlimit) ) { //length=1
        //check we don't already have this column
        if(colCheck[tmpCol]!=-1) {
          continue;
        }

        colCheck[tmpCol] = delta;
        lOneRows[delta] = i;
        knownCol[delta] = tmpCol;
        delta++;
        knowns++;
      }
    }

    if(delta==0) break;

    i=0;
    while(i<knowns){
      this->swapRows(lOneRows[i]+1, i+1 + sizeOfT);
      col->swapRows(lOneRows[i], i + sizeOfT);
      i++;
    }
    this->copyColumnsFromRows();

    //swap correct columns
    i=0;
    while(i<knowns){

      //column we're swapping into doesn't contain something, or *us*
      //(ie. we're already in the right column)
      colCheck[knownCol[i]] = -1;
      if(colCheck[i + sizeOfT] !=-1 && colCheck[i + sizeOfT]!=i + sizeOfT) {
        //tell it where it went
        colCheck[knownCol[i]] = colCheck[i + sizeOfT];
        knownCol[colCheck[i + sizeOfT]] = knownCol[i];
      }
      colCheck[i + sizeOfT] = -1;

      this->swapColumns(knownCol[i]+1, i+1 + sizeOfT);
      ans->swapRows(knownCol[i], i + sizeOfT);

      i++;
    }
    this->copyRowsFromColumns();

    sizeOfT += delta;
    
  } while (delta>0);

  return sizeOfT;
}

//TODO: change so we input a list and get back a list of swapped pairs
//then this list can be applied to as many objects as we like
//(roughly implemented in m2)
int sparseMatrix::triangulate(columnVector* col, columnVector* ans, sparseMatrix* m2){

  int sizeOfT=0, tempSize=0, delta;

  int lOneRows[this->height];
  int rowSwap[this->height];
  int colSwap[this->width];
  int colCheck[this->height];
  int rowCheck[this->height];
  
  //assumes this->height and this->width are equal, hmm
  for(int i=0; i<this->width; i++) {  colSwap[i]=0; }
  for(int i=0; i<this->height; i++) { colCheck[i]=0; rowCheck[i]=-1;rowSwap[i]=0; lOneRows[i]=0;}

  printf("triangulate() is deprecated, use triangulate2() instead\n");

  //find all rows with degree 1
  do{
    //tempSize=sizeOfT;
    printf("%i\n", sizeOfT);
    delta = 0;
    for(int i=sizeOfT; i<this->height; i++) {
      //record the position of the element (this locates the column)
      list<int>::iterator iter = (*this->rows)[i]->begin();
      while(sizeOfT!=0 && iter != (*this->rows)[i]->end() && (*iter)<sizeOfT) iter++;
      
      if(iter==(*this->rows)[i]->end()) { //length=0
        if(sizeOfT==0) {
          printf("Triangulation failed, blank row detected\n");
          return -i-2;
        }
        continue;
      }
      int tmpCol = (*iter);
      iter++;
      if(iter==(*this->rows)[i]->end()) { //length=1
        //check we don't already have this column
        if(colCheck[tmpCol]!=0) {
          if(sizeOfT!=0)  continue;
          //else { printf("Row=%i, col=%i\n", i, tmpCol); return -1; }//we have a matrix with duplicate rows
        }

        colCheck[tmpCol]=tempSize;
        printf("Row=%i, col=%i\n", i, tmpCol);
        lOneRows[tempSize] = i;
        colSwap[tempSize] = tmpCol;
        tempSize++;
        delta++;
      }
    }
    printf("blah\n");

    if(delta==0) {break;}


    for(int i=sizeOfT; i < tempSize; i++) {
      //reorder these rows to put them first
      swapRows(lOneRows[i]+1, i+1);
      col->swapRows(lOneRows[i], i);
      //if(m2!=NULL) m2->swapRows(lOneRows[i]+1, i+1);
      //if(m2!=NULL) m2->swapColumns(lOneRows[i]+1, i+1);
    }

    printf("blah3\n");
    //copy columns from rows
    this->copyColumnsFromRows();
    //if(m2!=NULL) m2->copyColumnsFromRows();
    //if(m2!=NULL) m2->copyRowsFromColumns();
    printf("blah4\n");
    //printf("\nBlah\n");
    //this->prettyPrint();

    for(int i=sizeOfT; i < tempSize; i++) {
      //reorder these rows to put them first
      printf("a\n");
      if(colCheck[i]!=0 && colCheck[i]!=i) {
        printf("%i\n", colCheck[i]);
        colSwap[colCheck[i]]=colSwap[i];
      }
      printf("a2\n");
      swapColumns(colSwap[i]+1, i+1);
      ans->swapRows(colSwap[i], i);
      if(m2!=NULL) m2->swapColumns(colSwap[i]+1, i+1);
      //if(m2!=NULL) m2->swapRows(colSwap[i]+1, i+1);

      //this->copyRowsFromColumns();
      //clear the colcheck
      printf("b\n");
      colCheck[colSwap[i]]=0;
    }

    printf("blah2\n");

    //reorder the columns so all of these are first
    int blah=0;
    for(int i=0; i<this->width; i++) blah+=colCheck[i];

    //copy rows from columns
    this->copyRowsFromColumns();
    if(m2!=NULL) m2->copyRowsFromColumns();
    //if(m2!=NULL) m2->copyColumnsFromRows();

    sizeOfT+=delta;
    //(first approx to triangular)
    //record size of triangle

  } while(delta>0);

  return sizeOfT;

//////////////////////////
// Extra triangulation
//////////////////////////

  delta=0;
  for(int i=sizeOfT; i<this->width; i++) {
    if((*this->columns)[i]->size()==0) { printf("Empty column\n"); return -1;}
    int tmpRow = (*this->columns)[i]->front();
    lOneRows[i]=-1;
    if( tmpRow >= sizeOfT && rowCheck[tmpRow]==-1) {
      rowCheck[tmpRow] = i;
      //lOneRows[tempSize] = tmpRow;
      lOneRows[i] = tmpRow;
      delta++;
      tempSize++;
    }
  }
  //this->prettyPrint();
  
  printf("Extra triangulation: %i\n", delta);


  int j=0;
  for(int i=sizeOfT; i < this->height; i++) {
    //reorder these rows to put them first
    if(rowCheck[i]!=-1) {
      //printf("swap this column: %i with %i\n", rowCheck[i], sizeOfT+j);

      rowSwap[sizeOfT + j] = i;
      if(lOneRows[sizeOfT + j] != -1 && lOneRows[sizeOfT + j] != i){

        rowCheck[lOneRows[sizeOfT + j]] = rowCheck[i];

        int tRowCheck = lOneRows[rowCheck[i]];
        lOneRows[rowCheck[i]] = lOneRows[sizeOfT + j];
        lOneRows[sizeOfT + j] = tRowCheck;

      }
      swapColumns(rowCheck[i] +1, sizeOfT + j+1);
      ans->swapRows(rowCheck[i], sizeOfT + j);
      //if(m2!=NULL) m2->swapRows(rowCheck[i] +1, sizeOfT + j+1);
      if(m2!=NULL) m2->swapColumns(rowCheck[i] +1, sizeOfT + j+1);


      j++;
    }
      //swapRows(lOneRows[i]+1, i+1);
  }

  this->copyRowsFromColumns();
  if(m2!=NULL) m2->copyRowsFromColumns();
  //if(m2!=NULL) m2->copyColumnsFromRows();

  //this->prettyPrint();

  for(int i=sizeOfT; i<sizeOfT+j; i++) {
    //printf("swap row %i with %i\n", lOneRows[i]+1, i+1);
    //printf("%i\n", rowSwap[i]);
    swapRows(rowSwap[i]+1, i+1);
    col->swapRows(rowSwap[i], i);
    //if(m2!=NULL) m2->swapRows(rowSwap[i]+1, i+1);
    //if(m2!=NULL) m2->swapColumns(rowSwap[i]+1, i+1);
  }

  this->copyColumnsFromRows();
  //if(m2!=NULL) m2->copyColumnsFromRows();
  //if(m2!=NULL) m2->copyRowsFromColumns();

  sizeOfT += delta;

  return sizeOfT;

}


/*
{

  printf("Triangulation stats\n");

  int step[this->width];
  int where[this->height];

  for(int i=0; i<this->width; i++) where[i]=-1;

  int currMax=0, sum=0, hits=0;

  for(int i=0; i<this->width; i++) {
    //printf("i = %i\n", i);
    int j = getFirstColValue(i+1) - 1;
    sum += j;
    step[i] = j;
    if(where[j]==-1) {
      where[j] = i;
      hits++;
    }
    if(j>currMax) currMax = j;
  }

  printf("currMax = %i, sum = %i, hits = %i\n", currMax, sum, hits);

  //for(int i=0; i<this->width; i++) printf("%i ", where[i]);
  printf("\n");

  return sizeOfT;
}
*/

//this is a pointless function
//TODO: remove it
int sparseMatrix::checkForRepeats(int level) {
  printf("Not implemented yet\n");

  //check down to ``level'' for rows repeated or duplicated
  int ret;
  printf("Checking at level%i\n", 1);
  for(int i=0; i<this->height; i++) {
    for(int j=i+1; j<this->height; j++) {
      //compare row i with row j
      ret=compareRows(i,j);
      if(ret==1) {
        printf("%i, %i\n", i, j);
        return j;
      }
    }
  }
  if(level>1){

  printf("Checking at level%i\n", 2);
  for(int i=0; i<this->height; i++) {
    for(int j=i+1; j<this->height; j++) {
      for(int k=j+1; k<this->height; k++) {
        //compare row i with row j+k
        ret=compareRows2(i,j,k);
        if(ret==1) {
          printf("%i, %i, %i\n", i, j, k);
          return k;
        }
      }
    }
  }
  }

  return 0;
}

//this is a pointless function
//TODO: remove it
int sparseMatrix::compareRows(int i, int j){
  list<int>::iterator iter = (*this->rows)[i]->begin();
  list<int>::iterator iter2 = (*this->rows)[j]->begin();
  
  while(iter != (*this->rows)[i]->end() && iter2 != (*this->rows)[j]->end()) {
    if((*iter)!=(*iter2)) return -1;
    iter++; iter2++; //they were the same so increment both
  }
  if(iter == (*this->rows)[i]->end() && iter2 == (*this->rows)[j]->end()) return 1;
  else return -1;
}

//this is a pointless function
//TODO: remove it
int sparseMatrix::compareRows2(int i, int j, int k){
  list<int>::iterator iter = (*this->rows)[i]->begin();
  list<int>::iterator iter2 = (*this->rows)[j]->begin();
  list<int>::iterator iter3 = (*this->rows)[k]->begin();
  
  while(iter != (*this->rows)[i]->end() && iter2 != (*this->rows)[j]->end() && iter3 != (*this->rows)[k]->end()) {
    if((*iter)==(*iter2)){
      if((*iter)==(*iter3)) return -1;
      iter++; iter2++; //they were the same so increment both
    }
    else if((*iter)==(*iter3)){
      iter++; iter3++; //they were the same so increment both
    }
    else {
      return -1;
    }
  }

  if(iter == (*this->rows)[i]->end() && (iter2 != (*this->rows)[j]->end() && iter3 != (*this->rows)[k]->end  ())) return -1;

  if(iter2 == (*this->rows)[j]->end()) {
    while(iter != (*this->rows)[i]->end() && iter3 != (*this->rows)[k]->end()) {
      if((*iter)!=(*iter3)) return -1;
      iter++; iter3++; //they were the same so increment both
    }
  }
  else if(iter3 == (*this->rows)[k]->end()) {
    while(iter != (*this->rows)[i]->end() && iter2 != (*this->rows)[j]->end()) {
      if((*iter)!=(*iter2)) return -1;
      iter++; iter2++; //they were the same so increment both
    }
  }

  if(iter == (*this->rows)[i]->end() && iter2 == (*this->rows)[j]->end() && iter3 == (*this->rows)[k]->end()) return 1;
  else return -1;
}


////////////////////////////////////////////////////////////////////////
//  Matrix inversion functions
////////////////////////////////////////////////////////////////////////

//decompose this into lower triangular times upper triangular
//FIXME: buggy
int sparseMatrix::decompose(sparseMatrix* l, sparseMatrix* u, sparseMatrix* m2){

  bool error=false;

  //rearrange all the columns so there are 1s down the diagonals
  //if not possible then the matrix is singular
  list<int>::iterator iter;

  if((*this->rows)[0]->size()==0) {
    printf("Matrix is singular at row %i\n", 0);
    return 0;
  }
  else if((*this->rows)[0]->front()!=0) {
    this->swapColumns(1, (*this->rows)[0]->front());
    if(m2!=NULL) m2->swapColumns(1, (*this->rows)[0]->front());

    printf("Swapping %i with %i\n", (*this->rows)[0]->front(), 0);

    this->copyRowsFromColumns();
  }


  for(int i=1; i<this->height; i++){
    iter = (*this->rows)[i]->begin();
    //skip up to 
    while(iter != (*this->rows)[i]->end()) {
      if(*iter == i){
        //try and swap this *off* the diagonal
        
        //iter++;
        //if(iter == (*this->rows)[i]->end() && ) {printf("Matrix is singular at row %i\n", i); return -i; }
        //FIXME: not completely optimal...
        int swapi = (i+1==this->width ? i : i+2);
        this->swapColumns(swapi, i+1);
        if(m2!=NULL) m2->swapColumns(swapi, i+1);
        printf("Swapping %i with %i\n", swapi, i+1);
        this->copyRowsFromColumns();
        break;
      }
      if(*iter > i)  break; //good
      iter++;
    }

  }

  iter = (*this->rows)[0]->begin();
  while(iter != (*this->rows)[0]->end()){
    u->insert(1, (*iter)+1);
    iter++;
  }
  l->insert(1,1);

  for(int i=1; i<this->height; i++){
    l->insert(i+1,i+1);

    iter = (*this->rows)[i]->begin();
    for(int j=0; j<this->width; j++){

      list<int>::iterator iter2;
      if(j<i){
        iter2 = (*u->rows)[j]->begin();
        if((*iter2)!=j){
          //swap the row with one with a 
          //if(error!=true) this->swapRows(i, (*this->columns)[i]->front());
          //printf("Swapping rows %i and %i\n" ,i, (*this->columns)[i]->front());
          if(!error) fprintf(stderr, "Warning: can't decompose. Giving up at row %i\n",i);
          //return -i-1;
          error=true;

        }
      }

      list<int>::iterator iter3;

      int tmp = 0;
      iter2 = (*l->rows)[i]->begin();
      iter3 = (*u->columns)[j]->begin();

      while(iter2 != (*l->rows)[i]->end() && iter3 != (*u->columns)[j]->end()){
        if(j<i && (*iter2)>=j && (*iter3)>=j) break;

        if((*iter2)<(*iter3)) iter2++;
        else if((*iter2)>(*iter3)) iter3++;
        else {tmp++; iter2++; iter3++;}
      }
      
      if(iter != (*this->rows)[i]->end() && (*iter)==j) {tmp++; iter++;}

      if(tmp%2==1){
        if(j<i) l->insert(i+1,j+1);
        else u->insert(i+1,j+1);
      }

      
    }

  }

  return 0;
  /*
  //check that the decomposition is correct
  sparseMatrix *t = new sparseMatrix(this->width, this->height);

  l->multiply(u, t);
  if(!t->equals(this)) printf("ERROR, decomposition failed.\n");
  */

}

int sparseMatrix::slackInvert(columnVector *a, columnVector *b){
  //perform Gaussian elimination on this to get b=this^{-1}a
  //this can be over determined (height>width)

  sparseMatrix *m = new sparseMatrix(this->width, this->width);
  columnVector *c = new columnVector(this->width);

  //iteration i
  //find r = first non-zero column (first row element)
  //if r>i
    //swap column r with this column i
    //goto iteration i+1
  //else
    //add row r to row i
    //goto iteration i

  list<int>::const_iterator iter1;
  int val;
  int swri[this->height];
  int swrk[this->height];
  int j=0, k=0, ret=0;
  for(int i=0; i<this->height; i++) {
    iter1=(*this->rows)[i]->begin();
    if(iter1==(*this->rows)[i]->end()) {
      //printf("Row %i is blank\n", i);
      ret++;
      //delete the row
      c->push(j, 0);
      this->symbolOps++;
      continue;
    }
    
    val = *iter1;
    this->binaryOps++;
    if(val>j) {
      m->copyColumnsFromRows();
      this->copyColumnsFromRows();
      this->swapColumns(val+1, j+1);
      m->swapColumns(val+1, j+1);
      this->copyRowsFromColumns();
      m->copyRowsFromColumns();
      swri[k] = val; swrk[k] = j;
      k++;
    }
    else if(val<j) {
      this->addRow(i, (*m->rows)[val]);
      c->push(j, c->get(j) ^ c->get(val));
      this->symbolOps++;
      i--;
      continue;
    }
    //else val==j
    //printf("Inverted row %i\n", i);

    c->push(j, c->get(j) ^ a->get(i));
    this->symbolOps++;

    iter1=(*this->rows)[i]->begin();
    while(iter1!=(*this->rows)[i]->end()) {
      
      (*m->rows)[j]->push_back(*iter1);
      //m->insert(j+1, (*iter1)+1);
      this->binaryOps++;
      iter1++;
    }
    j++;
    if(j>=this->width) break;
  }

  if(j<this->width) ret=j-this->width;//printf("Underdetermined matrix\n");

  //now solve by back substitution
  m->pivotEliminate2(c, b);
  this->binaryOps+=m->getBinaryOpCount();
  this->symbolOps+=m->getSymbolOpCount();

  //swap back b
  for(int i=k-1; i>=0; i--) {
    b->swapRows(swri[i], swrk[i]);
    this->symbolOps++;
  }

  delete c;
  delete m;

  return ret;
}


////////////////////////////////////////////////////////////////////////
//  Matrix addition functions
////////////////////////////////////////////////////////////////////////

void sparseMatrix::add(sparseMatrix *m2, sparseMatrix *dest){ //mod 2 addition

  //check matrices are the same size
  if(!sameSize(m2)) return;
  if(!sameSize(dest)) return;

  //empty destination
  dest->clear();

  //mutually traverse rows, and add unmatched elements to new matrix
  for(int i=0; i<this->height; i++) {
    list<int>::const_iterator iter1=(*this->rows)[i]->begin();
    list<int>::const_iterator iter2=(*m2->rows)[i]->begin();
    while(iter1 != (*this->rows)[i]->end() && iter2 != (*m2->rows)[i]->end()) {
      if((*iter1)<(*iter2)) {
        //add to dest
        dest->insert((*iter1)+1, i+1);
        iter1++;
      }
      else if((*iter1)>(*iter2)){
        //add to dest
        dest->insert((*iter2)+1, i+1);
        iter2++;
      }
      else { //if((*iter1)==(*iter2)) {
        iter1++; iter2++;
      }
      
    }
    if(iter1 == (*this->rows)[i]->end()) {
      //copy remainder of row from m2
      while(iter2 != (*m2->rows)[i]->end()){
        dest->insert((*iter2)+1, i+1);
        iter2++;
      }
    }
    else if(iter2 == (*m2->rows)[i]->end()) {
      //copy remainder of row from this
      while(iter1 != (*this->rows)[i]->end()){
        dest->insert((*iter1)+1, i+1);
        iter1++;
      }
    }
  }
}

//store result in this
void sparseMatrix::add(sparseMatrix *m2){ //mod 2 addition

  //check matrices are the same size
  if(!sameSize(m2)) return;
  
  //mutually traverse rows, and add unmatched elements to this matrix
  for(int i=0; i<this->height; i++) {
    list<int>::iterator iter1=(*this->rows)[i]->begin();
    list<int>::iterator iter2=(*m2->rows)[i]->begin();

    while(iter1 != (*this->rows)[i]->end() && iter2 != (*m2->rows)[i]->end()) {
      if((*iter1)<(*iter2)) {
        //do nothing
        iter1++;
      }
      else if((*iter1)>(*iter2)){
        //add to this
        iter1 = (*this->rows)[i]->insert(iter1, (*iter2));
        this->binaryOps++;

        //iter1->
        iter2++;
      }
      else { //if((*iter1)==(*iter2)) {
        //delete this point
        
        //list<int>->erase returns iterator to previous item
        iter1 = (*this->rows)[i]->erase(iter1);
        this->binaryOps++;
        
        //iter1++;
        iter2++;
      }
    }


    if(iter1 == (*this->rows)[i]->end()) {
      //copy remainder of row from m2
      while(iter2 != (*m2->rows)[i]->end()){

        (*this->rows)[i]->insert(iter1, (*iter2));
        this->binaryOps++;
        iter2++;
      }
    }
    else if(iter2 == (*m2->rows)[i]->end()) {
      //do nothing
    }
  }

  this->copyColumnsFromRows();
}

//store result in this
void sparseMatrix::add(sparseMatrix *m2, int srcrow, int where){ //mod 2 addition

  //check matrices are the same size
  //if(!sameSize(m2)) return;
  
  //mutually traverse rows, and add unmatched elements to new matrix

  list<int>::iterator iter1=(*this->rows)[where]->begin();
  list<int>::iterator iter2=(*m2->rows)[srcrow]->begin();

  while(iter1 != (*this->rows)[where]->end() && iter2 != (*m2->rows)[srcrow]->end()) {
    if((*iter1)<(*iter2)) {
        iter1++;
    }
    else if((*iter1)>(*iter2)){
      //add to dest
      (*this->rows)[where]->insert(iter1, (*iter2));
      iter2++;
    }
    else { //if((*iter1)==(*iter2)) {
      //delete this point
      iter1=(*this->rows)[where]->erase(iter1);
      iter2++;
    }
  }

  if(iter1 == (*this->rows)[where]->end()) {
    //copy remainder of row from m2
    while(iter2 != (*m2->rows)[srcrow]->end()){
      //dest->insert((*iter2)+1, i+1);
      (*this->rows)[where]->insert(iter1, (*iter2));
      iter2++;
    }
  }
  else if(iter2 == (*m2->rows)[srcrow]->end()) {
    //do nothing
  }

  //TODO: flag columns as dirty
  //this->copyColumnsFromRows();
}


////////////////////////////////////////////////////////////////////////
//  Matrix multiplication functions
////////////////////////////////////////////////////////////////////////

//dest=this*m2
void sparseMatrix::multiply2(const sparseMatrix *m2, sparseMatrix *dest){ //mod 2 multiplication
  printf("This function is deprecated. Use multiply(...) instead\n");
  //TODO: check this->width = m2->height
  //TODO: check dest is size m2->width*this->height

  //empty destination
  dest->clear();
  
  //walk this->rows, multiply by each m2->columns, result in dest
  for(int i=0; i<this->height; i++) {
    list<int>::const_iterator iter1;//=this->rows[i].begin();
    for(int j=0; j<m2->width; j++) {
      iter1=(*this->rows)[i]->begin();
      list<int>::const_iterator iter2=(*m2->columns)[j]->begin();
      int result=0;
      while(iter1 != (*this->rows)[i]->end() && iter2 != (*m2->columns)[j]->end()) {

        if((*iter1)<(*iter2)) {
          iter1++;
        }
        else if((*iter1)>(*iter2)){
          iter2++;
        }
        else { //if((*iter1)==(*iter2)) {
          iter1++; iter2++;
          result++;
        }
      }

      if((result%2)==1) { (*dest->rows)[i]->push_back(j);}
    }
  
  }

  //TODO: flag columns as dirty
  dest->copyColumnsFromRows();
}

//dest=this*m2
void sparseMatrix::multiply(const sparseMatrix *m2, sparseMatrix *dest){ //mod 2 multiplication
  //TODO: check this->width = m2->height
  //TODO: check dest is size m2->width*this->height

  //empty destination
  dest->clear();

  int rowWeights[this->height];
  
  //walk this->rows, multiply by each m2->columns, result in dest
  for(int i=0; i<m2->width; i++) {
    //clear rowWeights[]
    for(int k=0; k<this->height; k++) rowWeights[k]=0;
    list<int>::const_iterator iter1;//=this->rows[i].begin();
    //for(int j=0; j<m2->width; j++) {
    iter1=(*m2->columns)[i]->begin();
    while(iter1!=(*m2->columns)[i]->end()) {
      list<int>::const_iterator iter2=(*this->columns)[*iter1]->begin();
      while(iter2!=(*this->columns)[*iter1]->end()) {
        this->binaryOps++;

        rowWeights[*iter2]++;
        iter2++; 
      }
    

      iter1++;
    }
    for(int k=0; k<this->height; k++) {
      if(rowWeights[k]%2==1) { (*dest->columns)[i]->push_back(k);
        this->binaryOps++;
      }
    }
  }
  

  //TODO: flag columns as dirty
  dest->copyRowsFromColumns();
  //dest->copyColumnsFromRows();
}
//modulo 2 multiply...
void sparseMatrix::preMultiply(columnVector* v, columnVector* dest){

  //check v->getHeight()=this->width;

  //dest->clear();

  for(int i=0; i<this->height; i++) {
    list<int>::const_iterator iter=(*this->rows)[i]->begin();
    int result=0;
    while(iter != (*this->rows)[i]->end()){
      //result += v->get((*iter));
      result ^= v->get((*iter));
      this->symbolOps++; this->binaryOps++;
      iter++;
    }
    dest->push(i, result);
  }
}

//modulo 2 multiply...
void sparseMatrix::postMultiply(columnVector* v, columnVector* dest){

  //check v->getHeight()=this->width;

  //dest->clear();

  for(int i=0; i<this->width; i++) {
    list<int>::const_iterator iter=(*this->columns)[i]->begin();
    int result=0;
    while(iter != (*this->columns)[i]->end()){
      //result += v->get((*iter));
      result ^= v->get((*iter));
      iter++;
    }
    dest->push(i, result);
  }
}


////////////////////////////////////////////////////////////////////////
//  Elimination functions
////////////////////////////////////////////////////////////////////////

void sparseMatrix::pivotEliminate(columnVector* input, columnVector* result){
  
  //check input and result and this are all the correct size (square this?)

  //this must be lower triangular (how to check this?)
  //run over columns, checking getFirstColValue

  for(int i=0; i<this->height; i++){
    //copy over first value
    list<int>::const_iterator iter=(*this->rows)[i]->begin();

    int temp = input->get(i); //FIXME: choose correct type, how is this defined? perhaps templated

    while(iter != (*this->rows)[i]->end() && (*iter)<i){
      temp = temp ^ result->get(*iter);
      this->symbolOps++;
      this->binaryOps++;
      iter++;
    }
    result->push(i, temp);
  }

}

void sparseMatrix::pivotEliminate2(columnVector* input, columnVector* result){
  
  //check input and result and this are all the correct size (square this?)

  //this must be upper triangular (how to check this?)
  //run over columns, checking getFirstColValue

  for(int i=this->height-1; i>=0; i--){
    //copy over first value
    list<int>::const_iterator iter=(*this->rows)[i]->end();
    iter--;

    this->symbolOps++; this->binaryOps++;
    
    int temp = input->get(i); //FIXME: choose correct type, how is this defined? perhaps templated

    while(iter != (*this->rows)[i]->begin() && (*iter)>=i){
      temp = temp ^ result->get(*iter);
      this->symbolOps++; this->binaryOps++;
      iter--;
    }
    result->push(i, temp);
  }

}

void sparseMatrix::pivotEliminate(sparseMatrix* input, sparseMatrix* result){

  //check input and result and this are all the correct size (square this?)

  //this must be lower triangular (how to check this?)

  result->clear();

  //go row by row
  for(int i=0; i<this->height; i++){
    list<int>::const_iterator iter=(*this->rows)[i]->begin();
    result->add(input, i, i);

    while(iter != (*this->rows)[i]->end() && (*iter)<i){
      //add each line to result
      //TODO: does this actually work?
      //seems to :)
      result->add(result, (*iter), i);
      iter++;
    }
  }

}

void sparseMatrix::pivotEliminate2(sparseMatrix* input, sparseMatrix* result){

  //check input and result and this are all the correct size (square this?)

  //this must be lower triangular (how to check this?)
  
  int ws[result->width];


  for(int j=0; j<result->width; j++) ws[j]=0;

  result->clear();


  for(int i=this->height-1; i>=0; i--) {
    list<int>::const_iterator iter = (*input->rows)[i]->begin();
    //copy input to ws
    while(iter!= (*input->rows)[i]->end()) {
      ws[*iter]++;
      iter++;
    }
    iter=(*this->rows)[i]->end();
    iter--;
    while(iter!=(*this->rows)[i]->begin()) {
      //walk along all those rows in result
      list<int>::const_iterator iter2=(*result->rows)[*iter]->begin();
      while(iter2!=(*result->rows)[*iter]->end()) {
        ws[*iter2]++;
        this->binaryOps++;
        iter2++;
      }
      iter--;
    }

    for(int j=0; j<result->width; j++) {
      if(ws[j]%2==1) {(*result->rows)[i]->push_back(j);
        this->binaryOps++;
      }
      ws[j]=0;
    }
    //result->copyRowsFromColumns();
   
  }

  result->copyColumnsFromRows();
  
}

void sparseMatrix::split(int x, int y, int w, int h, sparseMatrix* result){
  //TODO: check result is big enough

  //loop over each row we want
  for(int i=y-1; i<y+h-1; i++){
    list<int>::const_iterator iter=(*this->rows)[i]->begin();

    while(iter != (*this->rows)[i]->end() && (*iter)<(x-1)) iter++;

    while(iter != (*this->rows)[i]->end() && (*iter)<x+w-1){
      (*result->rows)[i-(y-1)]->push_back((*iter)-(x-1));
      iter++;
      this->binaryOps++;
    }

  }
  result->copyColumnsFromRows();
}

////////////////////////////////////////////////////////////////////////
//  Matrix information functions
////////////////////////////////////////////////////////////////////////

bool sparseMatrix::sameSize(sparseMatrix *m){
  if(this->width != m->getWidth()) return false;
  if(this->height != m->getHeight()) return false;
  return true;
}

bool sparseMatrix::empty(){
  for(int i=0;i<this->height;i++){
    if(!(*this->rows)[i]->empty()) return false;
  }
  return true;
}


////////////////////////////////////////////////////////////////////////
//  Display functions
////////////////////////////////////////////////////////////////////////

void sparseMatrix::prettyPrint(){
  //loop over the rows
  for(int i=0; i<this->height; i++) {
    list<int>::const_iterator iter=(*this->rows)[i]->begin();
    printf("|");
    for(int j=0; j < this->width; j++){
      if(iter != (*this->rows)[i]->end() && (*iter) == j){
        printf(" 1 ");
        iter++;
      }
      else printf(" 0 ");
    }
    printf("|\n");
  }
}

void sparseMatrix::prettyPrintRow(int i){
  list<int>::const_iterator iter=(*this->rows)[i]->begin();
  printf("|");
  for(int j=0; j < this->width; j++){
    if(iter != (*this->rows)[i]->end() && (*iter) == j){
      printf(" 1 ");
      iter++;
    }
    else printf(" 0 ");
  }
  printf("|\n");
}

void sparseMatrix::prettyPrintTranspose(){
  //loop over the columns
  for(int i=0; i<this->width; i++) {
    list<int>::const_iterator iter=(*this->columns)[i]->begin();
    printf("|");
    for(int j=0; j < this->height; j++){
      if(iter != (*this->columns)[i]->end() && (*iter) == j){
        printf(" 1 ");
        iter++;
      }
      else printf(" 0 ");
    }
    printf("|\n");
  }
}

void sparseMatrix::coordPrint(){
  //this should be renamed
//void sparseMatrix::xpmPrint(){
  //loop over the rows
  printf("\\* XPM *\\\n");
  printf("static char *plaid[] = {\n");
  printf("\"%i %i 2 1\",\n", this->width, this->height);
  printf("\"  c white \",\n");
  printf("\"# c black \"");
  for(int i=0; i<this->height; i++) {
    list<int>::const_iterator iter=(*this->rows)[i]->begin();
    //printf("|");
    printf(",\n\"");
    for(int j=0; j < this->width; j++){
      if(iter != (*this->rows)[i]->end() && (*iter) == j) {
      //if(){
      //printf("%i %i\n", j, this->height-i);
        printf("#");
        iter++;
      }
      //}
      else {
        printf(" ");
      }
    }
    printf("\"");
  }
  printf("\n};\n");
}

void sparseMatrix::debugPrint(){
  //loop over the rows
  for(int i=0; i<this->height; i++) {
    list<int>::const_iterator iter=(*this->rows)[i]->begin();
    printf("|");
    while(iter != (*this->rows)[i]->end()){
      printf(" %i ", (*iter));
      iter++;
    }
    printf("|\n");
  }
}

void sparseMatrix::debugPrintTranspose(){
  //loop over the columns
  for(int i=0; i<this->width; i++) {
    list<int>::const_iterator iter=(*this->columns)[i]->begin();
    printf("|");
    while(iter != (*this->columns)[i]->end()){
      printf(" %i ", (*iter));
      iter++;
    }
    printf("|\n");
  }
}


////////////////////////////////////////////////////////////////////////
//  Private helper functions
////////////////////////////////////////////////////////////////////////

//TODO: return different values for each error
int sparseMatrix::checkWithinBounds(int i, int j){
  if(i<=0 || i>this->height) return -1;
  if(j<=0 || j>this->width) return -1;
  return 0;
}

bool sparseMatrix::checkRowColumns(){
  printf("checkRowColumns not implemented\n");
  return true;

  //FIXME: this code is completely wrong
  for(int i=0; i<this->height; i++) {

    list<int>::const_iterator iter=(*this->rows)[i]->begin();

    while(iter != (*this->rows)[i]->end()) {
      //walk column, adding each element to the correct row
      (*columns)[(*iter)]->push_back(i);
      iter++;
    }
  }
}

void sparseMatrix::copyRowsFromColumns(){
  //printf("copy r from c\n");
  //clear rows
  clearRows();
  //iterate over columns
  for(int i=0; i<this->width; i++) {
    list<int>::const_iterator iter=(*this->columns)[i]->begin();
    while(iter != (*this->columns)[i]->end()) {
      //walk column, adding each element to the correct row
      (*this->rows)[(*iter)]->push_back(i);
      iter++;
      this->binaryOps++;
    }
  }
}

void sparseMatrix::copyRowsFromColumn(int where){
  printf("copyRowsFromColumn is a broken function\n");
  return;
  //FIXME: broken
  list<int>::const_iterator iter=(*this->columns)[where]->begin();
  while(iter != (*this->columns)[where]->end()) {
    //FIXME: not being as efficient as we should be
    this->insert((*iter), where);
    iter++;
  }

}

void sparseMatrix::copyColumnsFromRows(){
  //printf("copy c from r\n");
  //clear columns
  clearColumns();

  //iterate over rows
  for(int i=0; i<this->height; i++) {
    list<int>::const_iterator iter=(*this->rows)[i]->begin();
    while(iter != (*this->rows)[i]->end()) {
      //walk column, adding each element to the correct row
      (*columns)[(*iter)]->push_back(i);
      iter++;
      this->binaryOps++;
    }
  }
}

void sparseMatrix::copyColumnsFromRows(int where, int where2) {
  printf("Not reliable\n");
  //clear columns
  for(int i=where; i<this->width; i++) {
    list<int>::iterator iter=(*this->columns)[i]->begin();
    while(iter!=(*this->columns)[i]->end() && *iter<where) iter++;
    if(iter!=(*this->columns)[i]->end()) {
      list<int>::iterator iter_start=iter;
      while(iter!=(*this->columns)[i]->end() && *iter<where2) iter++;
      (*this->columns)[i]->erase(iter_start, iter--);
    }
  }

  //iterate over rows
  for(int i=where; i<this->height; i++) {
    list<int>::const_iterator iter=(*this->rows)[i]->begin();
    while(iter != (*this->rows)[i]->end()) {
      //walk column, adding each element to the correct row
      if(*iter<where) printf("Hmm: %i, %i\n", *iter, where);
      (*columns)[(*iter)]->push_back(i);
      iter++;
    }
  }
}

void sparseMatrix::copyColumnsFromRow(int where){
  printf("copyColumnsFromRow is a broken function\n");
  return;
  //FIXME: broken
  list<int>::const_iterator iter=(*this->rows)[where]->begin();
  while(iter != (*this->rows)[where]->end()) {
    //FIXME: not being as efficient as we should be
    this->insert(where+1, (*iter)+1);
    iter++;
  }
}

void sparseMatrix::clearRows() {
  for(int i=0; i<this->height; i++) {
    (*this->rows)[i]->clear();
  }
}

void sparseMatrix::clearColumns() {
  for(int i=0; i<this->width; i++) {
    (*this->columns)[i]->clear();
  }
}
