/* Copyright 2006 Oliver Madge
 * 
 * This file is part of RU-decoder.
 *
 * RU-decoder is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2 as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program in the file COPYING; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 *
 */

#ifndef SPARSE_H
#define SPARSE_H

#include "stdio.h"
#include <list>
#include <vector>
#include <algorithm>
#include "columnvector.h"
#include "random.h"
#include "util.h"


using namespace std;

/** \mainpage A sparse matrix class
 * Provides modulo 2 manipulations of sparse matrices.
 * More detailed description to follow here.
 */


/**
 *  \class sparseMatrix 
 * Sparse matrix class. Provides modulo 2 manipulations of sparse matrices.
 * More detailed description to follow here.
 */

class sparseMatrix {
  public:
    
    /**
     * A constructor.
     * Create a sparseMatrix of width w and height h.
     * @param w an integer argument.
     * @param h an integer argument.
     * @see ~sparseMatrix()
     */
    sparseMatrix(int w, int h);

    /**
     * A destructor.
     * Delete the sparse matrix and free its memory.
     */
    ~sparseMatrix();

    /**
     * Add a row to the sparse matrix.
     * @param where An integer argument. -1 is last row.
     * @param input Row data.
     * @see insertColumn()
     */
    void insertRow(int where, list<int> *input);

    //void deleteRow(int where);
 
    void replaceRow(int where, list<int> *input);

    void replaceRow(sparseMatrix *m2, int where, int srcrow);

    void addRow(int where, list<int> *input);

    void addRow(int where, int where2);

    /**
     * Add a column to the sparse matrix.
     * @param where An integer argument.
     * @param input Column data.
     * @see insertRow()
     */
    void insertColumn(int where, list<int> *input);

    /**
     * Get the first non-zero value in a column.
     * @param col An integer specifying the column.
     * @return The first non-zero value in the specified column.
     */
    //FIXME: won't work with an empty column
    int getFirstColValue(int col) {return (*columns)[col-1]->front()+1;};
    int getFirstRowValue(int row) {return (*rows)[row-1]->front()+1;};

    /**
     * Add a new value at the specified x,y coordinate.
     * @param x X coordinate.
     * @param y Y coordinate.
     * @return Error code.
     */
    int insert(int x, int y); //, int value);

    /**
     * Remove the value at the specified x,y coordinate.
     * @param x X coordinate.
     * @param y Y coordinate.
     * @return Error code.
     */
    int remove(int x, int y);

    /**
     * Clear the sparse matrix 
     * @see empty()
     */
    void clear();

    /**
     * Swap columns with rows.
     */
    void transpose();

    /**
     * Swap row i with row j
     */
    void swapRows(int i, int j);

    /**
     * Swap row i with row j
     * Update columns automagically
     */
    void swapRowsSync(int i, int j);

    /**
     * Swap column i with column j
     */
    void swapColumns(int i, int j);

    //check if any columns are empty
    int blankColumns();

    //extremely slow function
    int determinant();

    //should be fast
    int lTDeterminant();
    int uTDeterminant();
    

    /**
     * Decompose the sparse matrix into lower and upper triangular parts.
     * Also known as LU decomposition. This is useful for inverting a matrix: once
     * l and u are known then successive applications of pivotEliminate and
     * pivotEliminate2 can be used to calculate the inverse.
     * @param l Pointer to a sparse matrix, will contain the lower triangular part.
     * @param u Pointer to a sparse matrix, will contain the upper triangular part.
     */
    int decompose(sparseMatrix* l, sparseMatrix* u, sparseMatrix* m2=NULL);

    int slackInvert(columnVector*, columnVector*);

    /**
     * Add another sparse matrix to this, and store the result elsewhere.
     * @param m2 Pointer to a sparse matrix to be added to this.
     * @param dest Pointer to a sparse matrix, result stored here.
     */
    void add(sparseMatrix *m2,sparseMatrix *dest);

    /**
     * Add another sparse matrix to this, and store the result in this.
     * @param m2 Pointer to a sparse matrix to be added to this.
     */
    void add(sparseMatrix *m2);

    /**
     * Add a row of another sparse matrix to a row of this.
     * @param m2 Pointer to a sparse matrix to be added to this.
     * @param srcrow Integer, which row to select from m2.
     * @param where Integer, which row in this to add to.
     */
    void add(sparseMatrix *m2, int srcrow, int where);

    /**
     * Multiply another sparse matrix by this, and store the result in dest.
     * @param m2 Pointer to a sparse matrix to be multiplied by this.
     * @param dest Pointer to a sparse matrix, result is stored here.
     */
    void multiply(const sparseMatrix *m2,sparseMatrix *dest);

    void multiply2(const sparseMatrix *m2,sparseMatrix *dest);

    void preMultiply(columnVector*, columnVector*);
    void postMultiply(columnVector*, columnVector*);

    /**
     * Select a region of this.
     * @param x X coordinate of selection.
     * @param y Y coordinate of selection.
     * @param w Width of selection.
     * @param h Height of selection.
     * @param result Pointer to a sparse matrix, result is stored here.
     */
    void split(int x, int y, int w, int h, sparseMatrix* result);

    void pivotEliminate(columnVector*, columnVector*);
    void pivotEliminate2(columnVector*, columnVector*);
    void pivotEliminate(sparseMatrix*, sparseMatrix*);
    void pivotEliminate2(sparseMatrix*, sparseMatrix*);

    int triangulate(columnVector*, columnVector*, sparseMatrix* m2=NULL);

    int triangulate2(columnVector*, columnVector*, Random*, sparseMatrix* m2=NULL);
    int triangulate3(columnVector*, columnVector*, Random*, sparseMatrix* m2=NULL);

    int triangulate4(columnVector*, columnVector*, Random*);
    
    int triangulate5(columnVector*, columnVector*, Random*, int);
    //void pivotEliminate();

    /**
     * Display the sparse matrix in a nice way.
     * Prints each row padded with zeros, so it won't be readable for
     * huge sparse matrices.
     * @see prettyPrintTranspose()
     * @see debugPrint()
     * @see debugPrintTranspose()
     */
    int checkForRepeats(int level);
    int compareRows(int i, int j);
    int compareRows2(int i, int j, int k);

    void prettyPrintRow(int i);
    void prettyPrint();
    
    void coordPrint();

    /**
     * Display the transpose of the sparse matrix in a nice way.
     * Prints each column padded with zeros, so it won't be readable for
     * huge sparse matrices.
     * @see prettyPrint()
     * @see debugPrint()
     * @see debugPrintTranspose()
     */
    void prettyPrintTranspose();

    /**
     * Display the sparse matrix in a form suitable for debugging.
     * Prints each row as a list of non-zero elements.
     * @see prettyPrint()
     * @see prettyPrintTranspose()
     * @see debugPrintTranspose()
     */
    void debugPrint();

    /**
     * Display the transpose of the sparse matrix in a form suitable for debugging.
     * Prints each column as a list of non-zero elements.
     * @see prettyPrint()
     * @see prettyPrintTranspose()
     * @see debugPrint()
     */
    void debugPrintTranspose();

    /**
     * Get the width of the sparse matrix.
     * @see getHeight()
     * @return The width
     */
    int getWidth() {return this->width;};

    /**
     * Get the height of the sparse matrix.
     * @see getWidth()
     * @return The height
     */
    int getHeight() {return this->height;};

    /**
     * Compares the size of the sparse matrix with another.
     * @param m sparse matrix
     * @see getWidth()
     * @see getHeight()
     * @return true if they are the same size
     */
    bool sameSize(sparseMatrix* m);

    /**
     * Checks if the sparse matrix is empty.
     * @see getWidth()
     * @see getHeight()
     * @return true if the sparse matrix is empty
     */
    bool empty();

    //all public functions must return consistent row/column representations
    //if an algorithm works on just one or the other then these functions must
    //be used to bring the two reps back in sync
    //they are quite slow though, so call them as infrequently as possible

    //FIXME: HACK! these should not be exposed
    void copyColumnsFromRows();
    void copyRowsFromColumns();

    void clearOpCounts() {this->symbolOps=0; this->binaryOps=0; this->swapOps=0;};
    int getBinaryOpCount() {return this->binaryOps;};
    int getSymbolOpCount() {return this->symbolOps;};
    int getSwapOpCount() {return this->swapOps;};


  protected:

    vector<list<int>* > *rows;
    vector<list<int>* > *columns;

    bool checkRowColumns(); //check if row and column representations are consistent

    void copyColumnsFromRow(int where);
    void copyColumnsFromRows(int where, int where2);

    void copyRowsFromColumn(int where);

  private:

    int checkWithinBounds(int i, int j);
    int splitDegreeRows(columnVector* col, int tlimit, int blimit);
    int splitDegreeRows(columnVector* col, int tlimit, int blimit, int*, int*, int*, int*, int*, int*, int*, int*, int*, int*, int*);
    int colSizeLim(int tlimit, int blimit, int i);


    int symbolOps, binaryOps, swapOps;

    int width, height;

    void clearRows();
    void clearColumns();
};


#endif
